/*
* This file is part of Bielefeld Type Library (BTL).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/btl
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
* 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
* 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
*/

/*
 * FaceIdentificationList.h
 *
 *  Created on: Nov 12, 2010
 *      Author: ltwardon
 */

#ifndef FACEIDENTIFICATIONLIST_H_
#define FACEIDENTIFICATIONLIST_H_

#include "Type.h"
#include <list>

namespace airobots
{

/**
 * List of all face identifications found.
 */
class FaceIdentificationList: public Type
{
public:

	/**
	 * One single result of the face identification.
	 */
	class FaceIdentification
	{
	private:
		int regionX;
		int regionY;
		int regionWidth;
		int regionHeight;
		int classId;

	public:
		FaceIdentification();

	    int getClassId() const;
	    int getRegionHeight() const;
	    int getRegionWidth() const;
	    int getRegionX() const;
	    int getRegionY() const;
	    void setClassId(int classId);
	    void setRegionHeight(int regionHeight);
	    void setRegionWidth(int regionWidth);
	    void setRegionX(int regionX);
	    void setRegionY(int regionY);
	    bool isIdentified() const;
	};

	FaceIdentificationList();

	static std::string getBaseTag();

	static std::string getTemplate();

	int size();

	std::list<FaceIdentification> getIdentifications();

	void sanitizeLocation(xmltio::Location &loc) const;
	virtual void fillInto(xmltio::Location &loc) const;
	static void fromLocation(const xmltio::Location &loc, FaceIdentificationList &type);

private:
	std::list<FaceIdentification> identifications;

	void addIdentification(FaceIdentification identification);
};

}

#endif /* FACEIDENTIFICATIONLIST_H_ */
