/*
* This file is part of Bielefeld Type Library (BTL).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/btl
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
* 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
* 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
*/

/*
 * KatanaPoseData.cpp
 *
 *  Created on: May 25, 2010
 *      Author: ttoenige
 */

#include "KatanaPoseData.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

std::string KatanaPoseData::getBaseTag()
{
	return "POSITION";
}

std::string KatanaPoseData::getTemplate()
{
	return std::string("<") + getBaseTag() + "><GENERATOR/>"
			+ Timestamp::getTemplate() + "<P x=\"\" y=\"\" z=\"\"/>"
			+ "<ANGLE phi=\"\" psi=\"\" theta=\"\" />" + "</" + getBaseTag()
			+ ">";
}

KatanaPoseData::KatanaPoseData() :
	Type("unknown")
{
	// TODO Auto-generated constructor stub

}

double KatanaPoseData::getX() const
{
	return x;
}
void KatanaPoseData::setX(double x)
{
	this->x = x;
}

double KatanaPoseData::getY() const
{
	return y;
}
void KatanaPoseData::setY(double y)
{
	this->y = y;
}

double KatanaPoseData::getZ() const
{
	return z;
}
void KatanaPoseData::setZ(double z)
{
	this->z = z;
}

double KatanaPoseData::getPhi() const
{
	return phi;
}
void KatanaPoseData::setPhi(double phi)
{
	this->phi = phi;
}

double KatanaPoseData::getPsi() const
{
	return psi;
}

void KatanaPoseData::setPsi(double psi)
{
	this->psi = psi;
}

double KatanaPoseData::getTheta() const
{
	return theta;
}
void KatanaPoseData::setTheta(double theta)
{
	this->theta = theta;
}

void KatanaPoseData::sanitizeLocation(xmltio::Location &loc) const
{

}

void KatanaPoseData::fillInto(xmltio::Location &loc) const
{

	Type::fillInto(loc);

	loc[XPath("P/@x")] = getX();
	loc[XPath("P/@y")] = getY();
	loc[XPath("P/@z")] = getZ();

	loc[XPath("ANGLE/@psi")] = getPsi();
	loc[XPath("ANGLE/@phi")] = getPhi();
	loc[XPath("ANGLE/@theta")] = getTheta();

}

void KatanaPoseData::fromLocation(const xmltio::Location &loc_,
		KatanaPoseData &kpd)
{

	Type::fromLocation(loc_, kpd);

	kpd.setX(extract<double> (loc_[XPath("P/@x")]));
	kpd.setY(extract<double> (loc_[XPath("P/@y")]));
	kpd.setZ(extract<double> (loc_[XPath("P/@z")]));

	kpd.setPhi(extract<double> (loc_[XPath("ANGLE/@phi")]));
	kpd.setPsi(extract<double> (loc_[XPath("ANGLE/@psi")]));
	kpd.setTheta(extract<double> (loc_[XPath("ANGLE/@theta")]));

}

