/*
* This file is part of Bielefeld Type Library (BTL).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/btl
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
* 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
* 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
*/

/*
 * Voice.cpp
 *
 *  Created on: Aug 21, 2009
 *      Author: jwienke
 */

#include "Voice.h"

using namespace std;
using namespace airobots;
using namespace xmltio;

Voice::Voice() :
	angle(0), runtimeDifference(0), angleVariance(0),
			runtimeDifferenceVariance(0) {
}

Voice::~Voice() {
}

std::string Voice::getBaseTag() {
	return "VOICE";
}

std::string Voice::getTemplate() {
	return string("<") + getBaseTag() + ">" + "<GENERATOR/>"
			+ Timestamp::getTemplate()
			+ "<POSITION angle=\"\" angleVariance=\"\" runtimeDifference=\"\" runtimeDifferenceVariance=\"\"/>"
				"</" + getBaseTag() + ">";
}

std::string Voice::getPerceptName() {
	return "voice";
}

float Voice::getAngle() const {
	return angle;
}

void Voice::setAngle(float angle) {
	this->angle = angle;
}

float Voice::getAngleVariance() const {
	return angleVariance;
}

void Voice::setAngleVariance(float angleVariance) {
	this->angleVariance = angleVariance;
}

float Voice::getRuntimeDifference() const {
	return runtimeDifference;
}

void Voice::setRuntimeDifference(float runtimeDifference) {
	this->runtimeDifference = runtimeDifference;
}

float Voice::getRuntimeDifferenceVariance() const {
	return runtimeDifferenceVariance;
}

void Voice::setRuntimeDifferenceVariance(float runtimeDifferenceVariance) {
	this->runtimeDifferenceVariance = runtimeDifferenceVariance;
}

bool Voice::operator==(const Voice &other) const {

	return Type::operator==(other) && (other.angle == angle)
			&& (other.runtimeDifference == runtimeDifference)
			&& (other.angleVariance == angleVariance)
			&& (other.runtimeDifferenceVariance == runtimeDifferenceVariance);

}

std::ostream& airobots::operator<<(std::ostream& o, Voice const& voice) {
	return o << "Voice [angle = " << voice.getAngle() << ", angleVariance = "
			<< voice.getAngleVariance() << ", runtimeDifference = "
			<< voice.getRuntimeDifference() << ", runtimeDifferenceVariance = "
			<< voice.getRuntimeDifferenceVariance() << "]";
}

void Voice::sanitizeLocation(xmltio::Location &loc) const {
	
}

void Voice::fillInto(xmltio::Location &loc) const {

	Type::fillInto(loc);

	loc[XPath("POSITION/@angle")] = getAngle();
	loc[XPath("POSITION/@angleVariance")] = getAngleVariance();

	loc[XPath("POSITION/@runtimeDifference")] = getRuntimeDifference();
	loc[XPath("POSITION/@runtimeDifferenceVariance")]
			= getRuntimeDifferenceVariance();

}

void Voice::fromLocation(const xmltio::Location &loc_, Voice &voice) {

	Type::fromLocation(loc_, voice);

	voice.setAngle(extract<float> (loc_[XPath("POSITION/@angle")]));
	voice.setAngleVariance(extract<float> (
			loc_[XPath("POSITION/@angleVariance")]));

	voice.setRuntimeDifference(extract<float> (loc_[XPath(
			"POSITION/@runtimeDifference")]));
	voice.setRuntimeDifferenceVariance(extract<float> (loc_[XPath(
			"POSITION/@runtimeDifferenceVariance")]));

}

