/*
* This file is part of Bielefeld Type Library (BTL).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/btl
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
* 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
* 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
*/

#include <boost/test/unit_test.hpp>
#include <boost/test/floating_point_comparison.hpp>

#include <iostream>

#include "btl/BinarySlamMap.h"
#include "utils.h"

using namespace airobots;
using namespace std;
using namespace xmltio;

static const double tolerance = 0.0001;

BOOST_AUTO_TEST_SUITE(BinarySlamMapTest)

BOOST_AUTO_TEST_CASE(testSelfCompatibility) {

	try {

		BinarySlamMap original;
		original.setGenerator("test");
		original.setTimestamp(Timestamp(123, 456));
		original.setOriginX(123);
		original.setOriginY(456);
		original.setResolution(0.23);
		original.setSizeX(34234);
		original.setSizeY(2345623);
		original.setTheta(-0.2);
		original.setUri("blubb/bla");
		original.setX(-34.56);
		original.setY(23.42);

		Location originalLoc(BinarySlamMap::getTemplate(), BinarySlamMap::getBaseTag());
		originalLoc = original;

		cout << originalLoc.getDocumentText(true);

		BinarySlamMap parsed = extract<BinarySlamMap>(originalLoc);

		BOOST_REQUIRE_EQUAL(parsed.getGenerator(), original.getGenerator());
		BOOST_REQUIRE_EQUAL(parsed.getTimestamp(), original.getTimestamp());
		BOOST_REQUIRE_EQUAL(parsed.getOriginX(), original.getOriginX());
		BOOST_REQUIRE_EQUAL(parsed.getOriginY(), original.getOriginY());
		BOOST_REQUIRE_CLOSE(parsed.getResolution(), original.getResolution(), tolerance);
		BOOST_REQUIRE_EQUAL(parsed.getSizeX(), original.getSizeX());
		BOOST_REQUIRE_EQUAL(parsed.getSizeY(), original.getSizeY());
		BOOST_REQUIRE_CLOSE(parsed.getTheta(), original.getTheta(), tolerance);
		BOOST_REQUIRE_EQUAL(parsed.getUri(), original.getUri());
		BOOST_REQUIRE_CLOSE(parsed.getX(), original.getX(), tolerance);
		BOOST_REQUIRE_CLOSE(parsed.getY(), original.getY(), tolerance);

	} catch (xmltio::XMLTIO_Exception e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());
	} catch (std::exception e) {
		BOOST_FAIL(string("Caught exception: ") + e.what());
	}

}

BOOST_AUTO_TEST_CASE(testFileCompatibility) {

	Location fileContents(readFile("BinarySlamMap.xml"), BinarySlamMap::getBaseTag());
	BinarySlamMap parsed = extract<BinarySlamMap>(fileContents);

	BOOST_REQUIRE_EQUAL("test", parsed.getGenerator());
	BOOST_REQUIRE_EQUAL(12345, parsed.getTimestamp().getInserted());
	BOOST_REQUIRE_EQUAL(67890, parsed.getTimestamp().getUpdated());
	BOOST_REQUIRE_EQUAL(74, parsed.getOriginX());
	BOOST_REQUIRE_EQUAL(34, parsed.getOriginY());
	BOOST_REQUIRE_CLOSE((float) 0.45, parsed.getResolution(), tolerance);
	BOOST_REQUIRE_EQUAL(745, parsed.getSizeX());
	BOOST_REQUIRE_EQUAL(123, parsed.getSizeY());
	BOOST_REQUIRE_CLOSE((float) 0.34, parsed.getTheta(), tolerance);
	BOOST_REQUIRE_EQUAL("test/uri", parsed.getUri());
	BOOST_REQUIRE_CLOSE((float) 234.5, parsed.getX(), tolerance);
	BOOST_REQUIRE_CLOSE((float) -6345.42, parsed.getY(), tolerance);

}

BOOST_AUTO_TEST_SUITE_END()
