/*
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * Copyright(c) Frederic Siepmann http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the GNU Lesser General Public
 * License Version 3 (the ``LGPL''), or (at your option) any later version.
 * 
 * Software distributed under the License is distributed on an ``AS IS'' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the LGPL for the
 * specific language governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this program. If not,
 * go to http://www.gnu.org/licenses/lgpl.html or write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 * 
 * The development of this software was supported by the Excellence Cluster EXC
 * 277 Cognitive Interaction Technology. The Excellence Cluster EXC 277 is a
 * grant of the Deutsche Forschungsgemeinschaft (DFG) in the context of the
 * German Excellence Initiative.
 * 
 * Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman
 * Köster, Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling, Denis
 * Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
 */
package de.unibi.airobots.btl.data.taskpattern;

import nu.xom.Element;
import nu.xom.Node;
import nu.xom.Nodes;
import de.unibi.airobots.btl.common.ElementParser;

/**
 * Send by the dialog to the system. system should handle this.
 * @author arothert, vrichter, bcarlmey
 */
public class LearnPerson extends TaskPatternBase {

	static final String NAME_ATT = "name";
	static final String ID_ATT = "id";

	private String name;
	private String id;

	public LearnPerson() {

	}

	/**
	 * the base tag should be named after the class itself.
	 * 
	 * @return
	 */
	public static String getBaseTag() {
		return "LEARN_PERSON";
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void fillInto(Element parent) {
		super.fillInto(parent);
		Element statesElement = TaskPatternStatusTagData
				.createTaskStatusElement(getOrigin(), getValue());
		parent.appendChild(statesElement);
//		parent.addAttribute(new Attribute(ID_ATT, getId()));
//		parent.addAttribute(new Attribute(NAME_ATT, getName()));		
	}

	public static void fromElement(Element element, LearnPerson type) {

		type.setName(element.getAttributeValue(NAME_ATT));
		type.setId(element.getAttributeValue(ID_ATT));

		Nodes nodes = element.query("*");
		for (int i = 0; i < nodes.size(); i++) {
			Node node = nodes.get(i);
			if (node instanceof Element) {
				Element elem = (Element) node;
				if (elem.getLocalName().equals(getBaseTag())) {
					System.err.println("in baseTag: " +ElementParser.getAttributeValue(elem, NAME_ATT)+
							ElementParser.getAttributeValue(elem, ID_ATT));
//					type.setName(ElementParser.getAttributeValue(elem, NAME_ATT));
//					type.setId(ElementParser.getAttributeValue(elem, ID_ATT));	
					
				} 
				if (elem.getLocalName().equals(STATUS_TAG)) {
					type.setValue(ElementParser.getAttributeValue(elem,
							VALUE_ATT));
					type.setOrigin(ElementParser.getAttributeValue(elem,
							ORIGIN_ATT));

				}
			}
		}
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getName() {
		return name;
	}

	public void setId(String id) {
		this.id = id;
	}

	public String getId() {
		return id;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {
		StringBuilder strb = new StringBuilder();
		strb.append("[" + getBaseTag() + ": ");
//		strb.append("name: " + name + "; ");
//		strb.append("id: " + id + " ");
		strb.append("[STATUS: ");
		strb.append("value: " + value + "; ");
		strb.append("origin: " + origin + "]]");
		return strb.toString();
	}
}
