package de.unibi.airobots.btl.data.taskpattern;

import de.unibi.airobots.btl.common.ElementParser;
import nu.xom.Element;
import nu.xom.Node;
import nu.xom.Nodes;


/**
 * Send by the SimpleFollowMeTaskPatternStrategy. Dialog handles it.
 * Send if human is lost.
 * @author arothert, vrichter, bcarlmey
 */
public class RobotLostHumanNotification extends TaskPatternBase{
	

	
	public RobotLostHumanNotification() {
		
	}
	
	/**
	 * the base tag should be named after the class itself.
	 * @return
	 */
	public static String getBaseTag() {
		return "ROBOT_LOST_HUMAN_NOTIFICATION";
	}
	

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void fillInto(Element parent) {
		super.fillInto(parent);
		
		Element statesElement = 
			TaskPatternStatusTagData.createTaskStatusElement(getOrigin(), getValue());
		parent.appendChild(statesElement);			
	}
	
	public static void fromElement(Element element, RobotLostHumanNotification type) {
		//Type.fromElement(element, type);
		
		Nodes nodes = element.query("*");
		for (int i = 0; i < nodes.size(); i++) {
			Node node = nodes.get(i);
			if (node instanceof Element) {
				Element elem = (Element) node;
				if (elem.getLocalName().equals(STATUS_TAG)) {
					type.setValue(ElementParser.getAttributeValue(elem, VALUE_ATT));
					type.setOrigin(ElementParser.getAttributeValue(elem, ORIGIN_ATT));			
				} 
			}
		}
	}
	
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {
		StringBuilder strb = new StringBuilder();
		strb.append("["+getBaseTag()+": ");
		strb.append("[STATUS: ");
		strb.append("value: " + getValue() + "; ");
		strb.append("origin: " + getOrigin() + "]] ");
		return strb.toString();
	}
	
}
