/*
* This file is part of Bielefeld Type Library (BTL).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/btl
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman Köster,
* 				Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling,
* 				Denis Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
*/
package de.unibi.airobots.btl.data;

import static org.junit.Assert.*;
import nu.xom.Builder;
import nu.xom.Document;
import nu.xom.Element;

import org.apache.log4j.Logger;
import org.junit.Test;

import de.unibi.airobots.btl.TestUtils;
import de.unibi.airobots.btl.common.Timestamp;

/**
 * Test cases for {@link Type}.
 * 
 * @author Denis Schulze <dschulze@techfak.uni-bielefeld.de>
 */
public class PersonListTest {

	private Logger logger = Logger.getLogger(getClass());

	@Test
	public void selfCompatibility() {

		PersonData data1 = new PersonData();

		final long timestamp = System.currentTimeMillis();
		data1.setTimestamp(timestamp);
		data1.setGenerator("test");
		data1.setAngle(-0.345);
		data1.setDistance(21.344);
		data1.setFacing(true);
		data1.setGazing(4.993);
		data1.setId(1234);
		data1.setName("peter");
		data1.setNeighbourDistance(3.45);
		data1.setNeighbourId(42);
		data1.setPoi(false);
		data1.setTalking(true);
		data1.setWalking(false);

		PersonData data2 = new PersonData();

		final long timestamp2 = System.currentTimeMillis();
		data2.setTimestamp(timestamp2);
		data2.setGenerator("test2");
		data2.setAngle(-0.542);
		data2.setDistance(12.3);
		data2.setFacing(false);
		data2.setGazing(410.45);
		data2.setId(4321);
		data2.setName("hans");
		data2.setNeighbourDistance(5.00);
		data2.setNeighbourId(1);
		data2.setPoi(true);
		data2.setTalking(false);
		data2.setWalking(true);

		PersonDataList original = new PersonDataList();
		original.addPersonData(data1);
		original.addPersonData(data2);

		Document doc = new Document(new Element(PersonDataList.getBaseTag()));
		original.fillInto(doc.getRootElement());
		logger.info(doc.toXML());

		PersonDataList parsed = new PersonDataList();
		PersonDataList.fromElement(doc.getRootElement(), parsed);
		
		PersonData parsed1 = parsed.getPersonData(0);
		PersonData parsed2 = parsed.getPersonData(1);

		assertEquals(data1.getGenerator(), parsed1.getGenerator());
		assertEquals(data1.getTimestamp(), parsed1.getTimestamp());
		assertEquals(data1.getAngle(), parsed1.getAngle(), 0.0001);
		assertEquals(data1.getDistance(), parsed1.getDistance(), 0.00001);
		assertEquals(data1.isFacing(), parsed1.isFacing());
		assertEquals(data1.getGazing(), parsed1.getGazing(), 0.00001);
		assertEquals(data1.getId(), parsed1.getId());
		assertEquals(data1.getName(), parsed1.getName());
		assertEquals(data1.getNeighbourDistance(), parsed1.getNeighbourDistance(), 0.0001);
		assertEquals(data1.getNeighbourId(), parsed1.getNeighbourId());
		assertEquals(data1.isPoi(), parsed1.isPoi());
		assertEquals(data1.isTalking(), parsed1.isTalking());
		assertEquals(data1.isWalking(), parsed1.isWalking());

		assertEquals(data2.getGenerator(), parsed2.getGenerator());
		assertEquals(data2.getTimestamp(), parsed2.getTimestamp());
		assertEquals(data2.getAngle(), parsed2.getAngle(), 0.0001);
		assertEquals(data2.getDistance(), parsed2.getDistance(), 0.00001);
		assertEquals(data2.isFacing(), parsed2.isFacing());
		assertEquals(data2.getGazing(), parsed2.getGazing(), 0.00001);
		assertEquals(data2.getId(), parsed2.getId());
		assertEquals(data2.getName(), parsed2.getName());
		assertEquals(data2.getNeighbourDistance(), parsed2.getNeighbourDistance(), 0.0001);
		assertEquals(data2.getNeighbourId(), parsed2.getNeighbourId());
		assertEquals(data2.isPoi(), parsed2.isPoi());
		assertEquals(data2.isTalking(), parsed2.isTalking());
		assertEquals(data2.isWalking(), parsed2.isWalking());

	}

	@Test
	public void fileCompatibility() throws Exception {

		Builder parser = new Builder();
		Document doc = parser.build(TestUtils
				.makeTestFileName("PersonList.xml"));

		PersonDataList parsed = new PersonDataList();
		PersonDataList.fromElement(doc.getRootElement(), parsed);
		assertEquals(doc.getRootElement().getLocalName(), PersonDataList.getBaseTag());

		PersonData parsed1 = parsed.getPersonData(0);

		assertEquals("test", parsed1.getGenerator());
		assertEquals(new Timestamp(127, 1271), parsed1.getTimestamp());
		assertEquals(-0.345, parsed1.getAngle(), 0.0001);
		assertEquals(21.344, parsed1.getDistance(), 0.00001);
		assertEquals(true, parsed1.isFacing());
		assertEquals(4.993, parsed1.getGazing(), 0.00001);
		assertEquals(1234, parsed1.getId());
		assertEquals("peter", parsed1.getName());
		assertEquals(3.45, parsed1.getNeighbourDistance(), 0.0001);
		assertEquals(42, parsed1.getNeighbourId());
		assertEquals(false, parsed1.isPoi());
		assertEquals(true, parsed1.isTalking());
		assertEquals(false, parsed1.isWalking());

	}

}
